/********************************************************************\
 * duigtktop.c -- top level gtk anchor point                        *
 * Copyright (C) 2002,2004 Linas Vepstas <linas@linas.org>          *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

#include "interface.h"
#include "window.h"
#include "perr.h"

struct DuiGtkTop_s
{
	GList *window_list;
};

/* =================================================================== */

DuiGtkTop *
dui_gtk_top_new (void)
{
	DuiGtkTop *top;

	top = g_new0 (DuiGtkTop, 1);
	top->window_list = NULL;
	return top;
}

void
dui_gtk_top_destroy (DuiGtkTop *top)
{
	GList *node;
	if (!top) return;
	for (node=top->window_list; node; node=node->next)
	{
		DuiWindow *win = node->data;
		dui_window_destroy (win);
	}
	g_list_free (top->window_list);
	g_free (top);
}

/* =================================================================== */

void 
dui_gtk_top_add_window (DuiGtkTop *top, DuiWindow *win)
{
	if (!win) return;
	top->window_list = g_list_append (top->window_list, win);
}

/* =================================================================== */

DuiWindow *
dui_gtk_top_find_window_by_name (DuiGtkTop *top, const char * name)
{
	GList *node;

	if (!name) return NULL;

	for (node=top->window_list; node; node=node->next)
	{
		DuiWindow *win = node->data;
		const char *winname = dui_window_get_name (win);
		if (winname && !strcmp(winname, name))
		{
			return win;
		}
	}
	return NULL;
}

/* =================================================================== */

static void 
dui_gtk_top_realize (gpointer self, gboolean fatal_if_no_main)
{
	DuiGtkTop *top = self;
	GList *node;
	gboolean found_app_window = 0;

	for (node=top->window_list; node; node=node->next)
	{
		DuiWindow *win = node->data;
		dui_window_resolve (win);
	}

	for (node=top->window_list; node; node=node->next)
	{
		DuiWindow *win = node->data;
		gboolean is_app_window = dui_window_is_app_main_window(win);
		if (is_app_window) 
		{
			dui_window_realize (win);
			found_app_window = 1;
		}
	}

	/* Without this test, its just to easy to write a dwi file 
	 * that just hangs there, doing nothing, and you scratch your 
	 * head wondering why ...
	 */
	if (fatal_if_no_main && 0 == found_app_window)
	{
		FATAL ("could not find an application main window!");
	}
}

/* =================================================================== */

DuiInterfacePlugin *
dui_gtk_top_plugin_new (DuiGtkTop *top)
{
	DuiInterfacePlugin *dip;

	dip = g_new0 (DuiInterfacePlugin, 1);
	dip->plugin_name = "dui_gtk_top";
	dip->self = top;
	dip->realize = dui_gtk_top_realize;

	return dip;
}

/* ========================== END OF FILE ============================ */
