
/*
 *  My Application
 *  Prototype for a new QOF DWI backend .... under development.
 *
 * Basic parsing works basic, copyin, copyout works.
 */

#define HAVE_QOF

#include <qof/qof.h>
#include <qof/qofinstance-p.h>
#include "qofmap.h"
#include "parse-qmap.h"

#include "database.h"
#include "duifield-qof.h"
#include "duifield-sql.h"
#include "duifieldmap.h"
#include "duiresolver.h"
#include "duitxnquery.h"
#include "duitxnreport.h"
#include "interface.h"
#include "perr.h"
#include "readfile.h"

#include "my-instance.h"

/* ============================================================== */
/* Perform various bits of system initialization */

GList *map_list = NULL;

void 
init (void)
{
	qof_object_initialize();
	qof_query_init();
	my_inst_register ();

	DuiInterface *di = dui_interface_new();
	DuiParser *par = dui_parser_new (di);
	// qof_parser_new (par);

	/* Register the needed plugins */
	QmapParseCtxt *ctx = qmap_new (par);
	dui_parse (par, "my-instance.dui");
	dui_parser_destroy (par);
	dui_interface_destroy(di);

	map_list = qmap_get_maplist (ctx);
}

void 
shutdown(void)
{
}

/* ============================================================== */
/* print out the contents of the book. */

static void
print_one (QofEntity *myent, gpointer data)
{
	const char * str = qof_object_printable (MYINST_ID, myent);
	printf ("%s", str);
}

void
print_all (QofBook *book)
{
	qof_object_foreach (MYINST_ID, book, print_one, NULL);
	printf ("-------------------------------------------------\n");
}

/* ============================================================== */

MyInst *
add_entity (QofBook *book)
{
	MyInst *me = my_inst_new (book);

	my_inst_set_a (me, 1234);

	time_t now = time(0);
	my_inst_set_b (me, now);

	char buff[300];
	sprintf (buff, "ZZZZZZZZZ! the time now is %s", ctime(&now));
	int len = strlen (buff);
	buff[len-1] = 0;  /* remove trailing newline char */
	my_inst_set_memo (me, buff);

	printf ("After adding some random entity, "
	        "the contents of the book now are:\n");
	print_all (book);
	return me;
}

int 
main (int argc, char *argv[])
{
	init();

	QofBook *book = qof_book_new();

	QofMap *map = map_list->data;
	qof_map_set_book (map, book);

	DuiDatabase *db = dui_database_new ("my db object", 
                          // "libdbi", "my-qof-db", 
                          // "odbc", "my-qof-db", 
                          "libpg", "my-qof-db", 
                          // "127.0.0.1", "linas", "abc123");
                          NULL, "linas", NULL);

	qof_map_set_database (map, db);

	/* -------------------------------------- */
	GUID guid;
   string_to_guid ("fedcba0123456789abcdef0123456789", &guid);
	qof_map_copy_from_db (map, &guid);

	/* Print all of the entities in the book, so that we can see 
	 * our handiwork. */
	printf ("After running the copy_from_db() routine, "
	        "the contents of the book are:\n");
	print_all (book);

	qof_map_copy_multiple_from_db (map, "select * from myinstance;");

	printf ("After running the copy_all_from_db() routine, "
	        "the contents of the book are:\n");
	print_all (book);

	/* -------------------------------------- */
	MyInst *me = add_entity (book);
	qof_map_write_to_db (map, QOF_INSTANCE (me));

	me->a = 42;
	qof_map_write_to_db (map, QOF_INSTANCE (me));

	/* Force a rollback to occur */
	me->a = 128;
	Timespec ts = qof_instance_get_last_update (&me->inst);
	ts.tv_nsec -=10;
	qof_instance_set_last_update (&me->inst, ts);
	qof_map_write_to_db (map, QOF_INSTANCE (me));

	shutdown ();

	return 0;
}

/* ======================== END OF FILE ========================= */
/* ============================================================== */
