/********************************************************************\
 * duiresolver.h -- Resolve names into actual pointers              *
 * Copyright (C) 2003 Linas Vepstas <linas@linas.org>               *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

/**
 * @file: duiresolver.h
 * @brief: Resolve names into actual pointers to actual objects 
 *
 * Fields are initially specified just as strings... that is,
 * names naming something.  At some point, these names have to
 * be converted into actual pointers to actual objects; that's what
 * this function does.
 *
 * @author: Copyright (c) 2003,2004 Linas Vepstas <linas@linas.org>
 */

#ifndef DUI_RESOLVER_H_
#define DUI_RESOLVER_H_

#include <glib.h>
#include "duifield.h"

typedef struct DuiResolver_s DuiResolver;

struct DuiResolver_s
{
	GList * field_list;
	GList * resolver_list;
	GList * realizer_list;
};

DuiResolver * dui_resolver_new (void);
void dui_resolver_destroy (DuiResolver *res);

/** Add a field which will need resolution later */
void dui_resolver_add_field (DuiResolver*, DuiField *);

typedef void (*DuiResolverFieldFunc) (DuiField *, gpointer);

/** Add a function which is able to resolve fields */
void dui_resolver_add_resolver (DuiResolver*, 
                                DuiResolverFieldFunc, 
                                gpointer user_data);

/** Remove function and its user_data from the resolver list. 
 *  Note: This routine is intended for cleanup/shutdown.  
 *  If you think you need this routine for any other purpose,
 *  you are wrong; what you really want is to use a different
 *  resolver function (and different user_data) that does what 
 *  you need.
 */
void dui_resolver_remove_resolver (DuiResolver*, 
                                DuiResolverFieldFunc, 
                                gpointer user_data);

typedef void (*DuiResolverRealizeFunc) (gpointer);

/** Add a function which, when called, will realize objects */
void dui_resolver_add_realizer (DuiResolver *, 
                                DuiResolverRealizeFunc, gpointer);

/** Do the deed. Call the DuiResolverFunc once on each field in the list
 * of fields.
 */
void dui_resolver_resolve (DuiResolver *);

void dui_resolver_realize (DuiResolver *);

#endif /* DUI_RESOLVER_H_ */
